﻿// ===== Собственные идентификаторы экземпляра класса F# =====

open System

//====================================================
// Подробное описание программного кода на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================


type MyClass(strdate: string) as classId =

    let mutable property = DateTime.Today

    let safeConvertData (str: string) : DateTime =
        try
            System.DateTime.Parse(str)

        // В случае исключения примем меры.
        with
        | ex ->
            printfn $"Не корректная дата!"
            // Изменение даты на прошлую не будет выводить
            // показания остатка дней до события.
            DateTime.Today.AddDays(-1)

    do
        // Идентификатору экземпляра класса доступны любые члены класса:
        // закрытые и открытые.
        classId.StringDate <- strdate
        classId.Date <- DateTime.Today.AddDays(1)
        classId.GetDaysBeforeDate()

    // Доступ невозможен! Области действия идентификаторов методов
    // только внутри члена класса
    //w.StringDate <- strdate
    //x.GetBeforeDate()

    // Если раскомментировать эту строку
    // возникнет исключение:
    // System.InvalidOperationException: "Инициализация объекта или значения привела к рекурсивному вызову объекта или значения перед его полной инициализацией."
    // do-привязка с использованием собственного идентификатора класса должна быть последней в определении
    // первичного конструктора.
    //let lastinit = 67

    // Собственный идентификатор метода необходим методу для доступа к другим членам класса.
    member m.StringDate
        with get () = property.ToLongDateString()
        and set (value) = property <- safeConvertData (value)

    member q.Date
        with private get () = property
        and private set (value) = property <- value

    member x.GetDaysBeforeDate() =
        let today = DateTime.Today
        // x.Date доступ к члену класса по собственному идентификатору метода.
        let before = (x.Date - today).Days

        if before >= 0 then
            printfn $"Осталось дней - %d{before} до даты %s{classId.StringDate}"



let id = MyClass("23 ноября 2021")

printfn $""
id.StringDate <- "2021.11.22"
id.GetDaysBeforeDate()

printfn $""
id.StringDate <- "2021.11.56"
id.GetDaysBeforeDate()
